from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.classification_options import ClassificationOptions


T = TypeVar("T", bound="ConfigPatchOptions")


@attr.s(auto_attribs=True)
class ConfigPatchOptions:
    """
    Attributes:
        classification (Union[Unset, ClassificationOptions]):
        collaborative_annotation (Union[Unset, bool]):
        image_filename (Union[Unset, str]):
        image_id (Union[Unset, str]):
        image_url (Union[Unset, str]):
        label (Union[Unset, str]):
        metafacets (Union[Unset, List[str]]):
    """

    classification: Union[Unset, "ClassificationOptions"] = UNSET
    collaborative_annotation: Union[Unset, bool] = UNSET
    image_filename: Union[Unset, str] = UNSET
    image_id: Union[Unset, str] = UNSET
    image_url: Union[Unset, str] = UNSET
    label: Union[Unset, str] = UNSET
    metafacets: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        classification: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.classification, Unset):
            classification = self.classification.to_dict()

        collaborative_annotation = self.collaborative_annotation
        image_filename = self.image_filename
        image_id = self.image_id
        image_url = self.image_url
        label = self.label
        metafacets: Union[Unset, List[str]] = UNSET
        if not isinstance(self.metafacets, Unset):
            metafacets = self.metafacets

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if classification is not UNSET:
            field_dict["classification"] = classification
        if collaborative_annotation is not UNSET:
            field_dict["collaborativeAnnotation"] = collaborative_annotation
        if image_filename is not UNSET:
            field_dict["imageFilename"] = image_filename
        if image_id is not UNSET:
            field_dict["imageId"] = image_id
        if image_url is not UNSET:
            field_dict["imageUrl"] = image_url
        if label is not UNSET:
            field_dict["label"] = label
        if metafacets is not UNSET:
            field_dict["metafacets"] = metafacets

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.classification_options import ClassificationOptions

        d = src_dict.copy()
        _classification = d.pop("classification", UNSET)
        classification: Union[Unset, ClassificationOptions]
        if isinstance(_classification, Unset):
            classification = UNSET
        else:
            classification = ClassificationOptions.from_dict(_classification)

        collaborative_annotation = d.pop("collaborativeAnnotation", UNSET)

        image_filename = d.pop("imageFilename", UNSET)

        image_id = d.pop("imageId", UNSET)

        image_url = d.pop("imageUrl", UNSET)

        label = d.pop("label", UNSET)

        metafacets = cast(List[str], d.pop("metafacets", UNSET))

        config_patch_options = cls(
            classification=classification,
            collaborative_annotation=collaborative_annotation,
            image_filename=image_filename,
            image_id=image_id,
            image_url=image_url,
            label=label,
            metafacets=metafacets,
        )

        return config_patch_options



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="AltText")


@attr.s(auto_attribs=True)
class AltText:
    """
    Attributes:
        count (int):
        name (str):
    """

    count: int
    name: str

    def to_dict(self) -> Dict[str, Any]:
        count = self.count
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "count": count,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        count = d.pop("count")

        name = d.pop("name")

        alt_text = cls(
            count=count,
            name=name,
        )

        return alt_text



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.item_count import ItemCount
    from ..models.sherpa_job_bean import SherpaJobBean


T = TypeVar("T", bound="DeleteManyResponse")


@attr.s(auto_attribs=True)
class DeleteManyResponse:
    """
    Attributes:
        removed (int):
        details (Union[Unset, List['ItemCount']]):
        job (Union[Unset, SherpaJobBean]):
    """

    removed: int
    details: Union[Unset, List["ItemCount"]] = UNSET
    job: Union[Unset, "SherpaJobBean"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        removed = self.removed
        details: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.details, Unset):
            details = []
            for details_item_data in self.details:
                details_item = details_item_data.to_dict()

                details.append(details_item)

        job: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.job, Unset):
            job = self.job.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "removed": removed,
            }
        )
        if details is not UNSET:
            field_dict["details"] = details
        if job is not UNSET:
            field_dict["job"] = job

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.item_count import ItemCount
        from ..models.sherpa_job_bean import SherpaJobBean

        d = src_dict.copy()
        removed = d.pop("removed")

        details = []
        _details = d.pop("details", UNSET)
        for details_item_data in _details or []:
            details_item = ItemCount.from_dict(details_item_data)

            details.append(details_item)

        _job = d.pop("job", UNSET)
        job: Union[Unset, SherpaJobBean]
        if isinstance(_job, Unset):
            job = UNSET
        else:
            job = SherpaJobBean.from_dict(_job)

        delete_many_response = cls(
            removed=removed,
            details=details,
            job=job,
        )

        return delete_many_response



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

if TYPE_CHECKING:
    from ..models.input_document import InputDocument
    from ..models.with_annotator import WithAnnotator
    from ..models.with_converter import WithConverter
    from ..models.with_language_guesser import WithLanguageGuesser
    from ..models.with_processor import WithProcessor
    from ..models.with_segmenter import WithSegmenter


T = TypeVar("T", bound="AnnotateDocumentsWithPipeline")


@attr.s(auto_attribs=True)
class AnnotateDocumentsWithPipeline:
    """
    Attributes:
        documents (List['InputDocument']):
        pipeline (List[Union['WithAnnotator', 'WithConverter', 'WithLanguageGuesser', 'WithProcessor',
            'WithSegmenter']]):
    """

    documents: List["InputDocument"]
    pipeline: List[Union["WithAnnotator", "WithConverter", "WithLanguageGuesser", "WithProcessor", "WithSegmenter"]]

    def to_dict(self) -> Dict[str, Any]:
        from ..models.with_annotator import WithAnnotator
        from ..models.with_language_guesser import WithLanguageGuesser
        from ..models.with_processor import WithProcessor
        from ..models.with_segmenter import WithSegmenter

        documents = []
        for documents_item_data in self.documents:
            documents_item = documents_item_data.to_dict()

            documents.append(documents_item)

        pipeline = []
        for pipeline_item_data in self.pipeline:
            pipeline_item: Dict[str, Any]

            if isinstance(pipeline_item_data, WithAnnotator):
                pipeline_item = pipeline_item_data.to_dict()

            elif isinstance(pipeline_item_data, WithProcessor):
                pipeline_item = pipeline_item_data.to_dict()

            elif isinstance(pipeline_item_data, WithLanguageGuesser):
                pipeline_item = pipeline_item_data.to_dict()

            elif isinstance(pipeline_item_data, WithSegmenter):
                pipeline_item = pipeline_item_data.to_dict()

            else:
                pipeline_item = pipeline_item_data.to_dict()

            pipeline.append(pipeline_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "documents": documents,
                "pipeline": pipeline,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.input_document import InputDocument
        from ..models.with_annotator import WithAnnotator
        from ..models.with_converter import WithConverter
        from ..models.with_language_guesser import WithLanguageGuesser
        from ..models.with_processor import WithProcessor
        from ..models.with_segmenter import WithSegmenter

        d = src_dict.copy()
        documents = []
        _documents = d.pop("documents")
        for documents_item_data in _documents:
            documents_item = InputDocument.from_dict(documents_item_data)

            documents.append(documents_item)

        pipeline = []
        _pipeline = d.pop("pipeline")
        for pipeline_item_data in _pipeline:

            def _parse_pipeline_item(
                data: object,
            ) -> Union["WithAnnotator", "WithConverter", "WithLanguageGuesser", "WithProcessor", "WithSegmenter"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    pipeline_item_type_0 = WithAnnotator.from_dict(data)

                    return pipeline_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    pipeline_item_type_1 = WithProcessor.from_dict(data)

                    return pipeline_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    pipeline_item_type_2 = WithLanguageGuesser.from_dict(data)

                    return pipeline_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    pipeline_item_type_3 = WithSegmenter.from_dict(data)

                    return pipeline_item_type_3
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                pipeline_item_type_4 = WithConverter.from_dict(data)

                return pipeline_item_type_4

            pipeline_item = _parse_pipeline_item(pipeline_item_data)

            pipeline.append(pipeline_item)

        annotate_documents_with_pipeline = cls(
            documents=documents,
            pipeline=pipeline,
        )

        return annotate_documents_with_pipeline



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="RoleUpdate")


@attr.s(auto_attribs=True)
class RoleUpdate:
    """
    Attributes:
        label (Union[Unset, str]):
        permissions (Union[Unset, List[str]]):
    """

    label: Union[Unset, str] = UNSET
    permissions: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        permissions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = self.permissions

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if label is not UNSET:
            field_dict["label"] = label
        if permissions is not UNSET:
            field_dict["permissions"] = permissions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label", UNSET)

        permissions = cast(List[str], d.pop("permissions", UNSET))

        role_update = cls(
            label=label,
            permissions=permissions,
        )

        return role_update



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.doc_annotation_creation_mode import DocAnnotationCreationMode
from ..models.doc_annotation_status import DocAnnotationStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.annotation_term import AnnotationTerm
    from ..models.doc_annotation_properties import DocAnnotationProperties


T = TypeVar("T", bound="DocAnnotation")


@attr.s(auto_attribs=True)
class DocAnnotation:
    """A document annotation

    Attributes:
        end (int): End offset in document
        label_name (str): The label name
        start (int): Start offset in document
        text (str): Covered text
        created_by (Union[Unset, str]): User having created the annotation
        created_date (Union[Unset, str]): Creation date
        creation_mode (Union[Unset, DocAnnotationCreationMode]): Creation mode
        identifier (Union[Unset, str]): Annotation identifier (only in 'html version')
        modified_date (Union[Unset, str]): Last modification date
        properties (Union[Unset, DocAnnotationProperties]): Additional properties
        status (Union[Unset, DocAnnotationStatus]): Status of the annotation
        terms (Union[Unset, List['AnnotationTerm']]):
    """

    end: int
    label_name: str
    start: int
    text: str
    created_by: Union[Unset, str] = UNSET
    created_date: Union[Unset, str] = UNSET
    creation_mode: Union[Unset, DocAnnotationCreationMode] = UNSET
    identifier: Union[Unset, str] = UNSET
    modified_date: Union[Unset, str] = UNSET
    properties: Union[Unset, "DocAnnotationProperties"] = UNSET
    status: Union[Unset, DocAnnotationStatus] = UNSET
    terms: Union[Unset, List["AnnotationTerm"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        end = self.end
        label_name = self.label_name
        start = self.start
        text = self.text
        created_by = self.created_by
        created_date = self.created_date
        creation_mode: Union[Unset, str] = UNSET
        if not isinstance(self.creation_mode, Unset):
            creation_mode = self.creation_mode.value

        identifier = self.identifier
        modified_date = self.modified_date
        properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = self.properties.to_dict()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        terms: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.terms, Unset):
            terms = []
            for terms_item_data in self.terms:
                terms_item = terms_item_data.to_dict()

                terms.append(terms_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "end": end,
                "labelName": label_name,
                "start": start,
                "text": text,
            }
        )
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if created_date is not UNSET:
            field_dict["createdDate"] = created_date
        if creation_mode is not UNSET:
            field_dict["creationMode"] = creation_mode
        if identifier is not UNSET:
            field_dict["identifier"] = identifier
        if modified_date is not UNSET:
            field_dict["modifiedDate"] = modified_date
        if properties is not UNSET:
            field_dict["properties"] = properties
        if status is not UNSET:
            field_dict["status"] = status
        if terms is not UNSET:
            field_dict["terms"] = terms

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotation_term import AnnotationTerm
        from ..models.doc_annotation_properties import DocAnnotationProperties

        d = src_dict.copy()
        end = d.pop("end")

        label_name = d.pop("labelName")

        start = d.pop("start")

        text = d.pop("text")

        created_by = d.pop("createdBy", UNSET)

        created_date = d.pop("createdDate", UNSET)

        _creation_mode = d.pop("creationMode", UNSET)
        creation_mode: Union[Unset, DocAnnotationCreationMode]
        if isinstance(_creation_mode, Unset):
            creation_mode = UNSET
        else:
            creation_mode = DocAnnotationCreationMode(_creation_mode)

        identifier = d.pop("identifier", UNSET)

        modified_date = d.pop("modifiedDate", UNSET)

        _properties = d.pop("properties", UNSET)
        properties: Union[Unset, DocAnnotationProperties]
        if isinstance(_properties, Unset):
            properties = UNSET
        else:
            properties = DocAnnotationProperties.from_dict(_properties)

        _status = d.pop("status", UNSET)
        status: Union[Unset, DocAnnotationStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = DocAnnotationStatus(_status)

        terms = []
        _terms = d.pop("terms", UNSET)
        for terms_item_data in _terms or []:
            terms_item = AnnotationTerm.from_dict(terms_item_data)

            terms.append(terms_item)

        doc_annotation = cls(
            end=end,
            label_name=label_name,
            start=start,
            text=text,
            created_by=created_by,
            created_date=created_date,
            creation_mode=creation_mode,
            identifier=identifier,
            modified_date=modified_date,
            properties=properties,
            status=status,
            terms=terms,
        )

        return doc_annotation
